<?php

namespace App\Http\Livewire\Tables;

use App\Helpers\LogHelper;
use App\Models\Task;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Database\Eloquent\Builder;
use Rappasoft\LaravelLivewireTables\Views\Column;
use Rappasoft\LaravelLivewireTables\DataTableComponent;

class TaskArchiveDatatables extends DataTableComponent
{
    protected $model = Task::class;

    public function configure(): void
    {
        $this->setPrimaryKey('id');
        $this->setEagerLoadAllRelationsEnabled();
    }

    public function builder(): Builder
    {
        return Task::query()
            ->onlyTrashed();
    }

    public function permanently_delete($id)
    {

        $task = Task::onlyTrashed()->findOrFail($id);
        $task_name = $task->title;
        $task->forceDelete();

        if (Auth::user()->userData) {
            $deleted_by = Auth::user()->userData->fullName();
        } else {
            $deleted_by = Auth::user()->username;
        }

        $message = __('messages.Task') . " $task_name " . __('messages.is Permanently Deleted Successfully');
        $data = ['task' => $task];
        LogHelper::logInfo($message, $data);

        return redirect()->to('/tasks-archived')->with('message', $message);
    }

    public function restore($id)
    {
        $task = Task::onlyTrashed()->findOrFail($id);
        $task->restore();
        $task->update(['deleted_by' => null]);

        $message = __('messages.Task') . " $task->title " . __('messages.is Restored Successfully');
        $data = ['task' => $task];
        LogHelper::logInfo($message, $data);

        return redirect()->to('/tasks/archived')->with('message', $message);
    }

    public function columns(): array
    {

        $colssums = [
            Column::make(__('messages.Id'), "id")
                ->hideIf(true)
                ->sortable(),
            Column::make(__('messages.Title'), "title")
                ->sortable()
                ->searchable(),
            Column::make(__('messages.Description'), "description")
                ->sortable()
                ->searchable(),
            Column::make(__('messages.Due date'), "due_date")
                ->sortable(),
            Column::make(__('messages.Created at'), "created_at")
                ->hideIf(true)
                ->sortable(),
            Column::make(__('messages.Updated at'), "updated_at")
                ->hideIf(true)
                ->sortable(),
        ];




        $final_colums =  [

            Column::make(__('messages.Deleted at'), "deleted_at")
                ->sortable(),
            Column::make(__('messages.Deleted by'), "deleted_by")
                ->sortable(),

            Column::make(__('messages.Actions'))
                ->label(
                    function ($row, Column $column) {
                        $permanently_delete = '
                            <button
                                class="inline-flex items-center text-white bg-red-700 hover:bg-red-800 focus:outline-none focus:ring-4 focus:ring-red-300 font-medium rounded-full text-sm h-10 px-5 m-2 text-center dark:bg-red-600 dark:hover:bg-red-700 dark:focus:ring-red-900"
                                type="button"
                                data-modal-toggle="delete-modal_' . $row->id . '"
                                >
                                <svg xmlns="http://www.w3.org/2000/svg" class="h-5 w-5 mr-1" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                                    <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M19 7l-.867 12.142A2 2 0 0116.138 21H7.862a2 2 0 01-1.995-1.858L5 7m5 4v6m4-6v6m1-10V4a1 1 0 00-1-1h-4a1 1 0 00-1 1v3M4 7h16" />
                                </svg>
                                ' . __('messages.Delete') . '
                            </button>
                            <div id="delete-modal_' . $row->id . '" tabindex="-1" class="hidden overflow-y-auto overflow-x-hidden fixed top-0 right-0 left-0 z-50 p-4 md:inset-0 h-modal md:h-full">
                                <div class="relative w-full max-w-md h-full md:h-auto">
                                    <div class="relative bg-white rounded-lg shadow dark:bg-gray-700">
                                        <button type="button" class="absolute top-3 right-2.5 text-gray-400 bg-transparent hover:bg-gray-200 hover:text-gray-900 rounded-lg text-sm p-1.5 ml-auto inline-flex items-center dark:hover:bg-gray-800 dark:hover:text-white" data-modal-toggle="delete-modal_' . $row->id . '">
                                            <svg aria-hidden="true" class="w-5 h-5" fill="currentColor" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg"><path fill-rule="evenodd" d="M4.293 4.293a1 1 0 011.414 0L10 8.586l4.293-4.293a1 1 0 111.414 1.414L11.414 10l4.293 4.293a1 1 0 01-1.414 1.414L10 11.414l-4.293 4.293a1 1 0 01-1.414-1.414L8.586 10 4.293 5.707a1 1 0 010-1.414z" clip-rule="evenodd"></path></svg>
                                            <span class="sr-only">
                                            ' . __('messages.Close modal') . '
                                            </span>
                                        </button>
                                        <div class="p-6 text-center">
                                            <svg aria-hidden="true" class="mx-auto mb-4 w-14 h-14 text-gray-400 dark:text-gray-200" fill="none" stroke="currentColor" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M12 8v4m0 4h.01M21 12a9 9 0 11-18 0 9 9 0 0118 0z"></path></svg>
                                            <h3 class="mb-5 text-lg font-normal text-gray-500 dark:text-gray-400">
                                            ' . __('messages.Are you sure you want to permanently delete this task?') . '
                                            </h3>
                                            <button data-modal-toggle="delete-modal_' . $row->id . '" type="button" class="text-white bg-red-600 hover:bg-red-800 focus:ring-4 focus:outline-none focus:ring-red-300 dark:focus:ring-red-800 font-medium rounded-lg text-sm inline-flex items-center px-5 py-2.5 text-center mr-2" wire:click="permanently_delete(' . $row->id . ')">
                                                ' . __('messages.Yes, I am sure') . '
                                            </button>
                                            <button data-modal-toggle="delete-modal_' . $row->id . '" type="button" class="text-gray-500 bg-white hover:bg-gray-100 focus:ring-4 focus:outline-none focus:ring-gray-200 rounded-lg border border-gray-200 text-sm font-medium px-5 py-2.5 hover:text-gray-900 focus:z-10 dark:bg-gray-700 dark:text-gray-300 dark:border-gray-500 dark:hover:text-white dark:hover:bg-gray-600 dark:focus:ring-gray-600">
                                            ' . __('messages.No, cancel') . '
                                            </button>
                                        </div>
                                    </div>
                                </div>
                            </div>';
                        $restore = '
                        <button
                            class="inline-flex items-center text-white bg-indigo-500 hover:bg-indigo-600 focus:outline-none focus:ring-4 focus:ring-indigo-300 font-medium rounded-full text-sm h-10 px-5 m-2 text-center dark:bg-indigo-900 dark:hover:bg-indigo-800 dark:focus:ring-indigo-900"

                            type="button"
                            data-modal-toggle="restore-modal"
                            >
                            <svg xmlns="http://www.w3.org/2000/svg" class="h-5 w-5 mr-1" fill="none" viewBox="0 0 24 24" stroke="currentColor">
                                <path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M3 10h10a8 8 0 018 8v2M3 10l6 6m-6-6l6-6" />
                            </svg>
                            ' . __('messages.Restore') . '
                        </button>
                            <div id="restore-modal" tabindex="-1" class="hidden overflow-y-auto overflow-x-hidden fixed top-0 right-0 left-0 z-50 p-4 md:inset-0 h-modal md:h-full">
                                <div class="relative w-full max-w-md h-full md:h-auto">
                                    <div class="relative bg-white rounded-lg shadow dark:bg-gray-700">
                                        <button type="button" class="absolute top-3 right-2.5 text-gray-400 bg-transparent hover:bg-gray-200 hover:text-gray-900 rounded-lg text-sm p-1.5 ml-auto inline-flex items-center dark:hover:bg-gray-800 dark:hover:text-white" data-modal-toggle="restore-modal">
                                            <svg aria-hidden="true" class="w-5 h-5" fill="currentColor" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg"><path fill-rule="evenodd" d="M4.293 4.293a1 1 0 011.414 0L10 8.586l4.293-4.293a1 1 0 111.414 1.414L11.414 10l4.293 4.293a1 1 0 01-1.414 1.414L10 11.414l-4.293 4.293a1 1 0 01-1.414-1.414L8.586 10 4.293 5.707a1 1 0 010-1.414z" clip-rule="evenodd"></path></svg>
                                            <span class="sr-only">
                                            ' . __('messages.Close modal') . '
                                            </span>
                                        </button>
                                        <div class="p-6 text-center">
                                            <svg aria-hidden="true" class="mx-auto mb-4 w-14 h-14 text-gray-400 dark:text-gray-200" fill="none" stroke="currentColor" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg"><path stroke-linecap="round" stroke-linejoin="round" stroke-width="2" d="M12 8v4m0 4h.01M21 12a9 9 0 11-18 0 9 9 0 0118 0z"></path></svg>
                                            <h3 class="mb-5 text-lg font-normal text-gray-500 dark:text-gray-400">
                                            ' . __('messages.Are you sure you want to restore this task?') . '
                                            </h3>
                                            <button data-modal-toggle="restore-modal" type="button" class="text-white bg-red-600 hover:bg-red-800 focus:ring-4 focus:outline-none focus:ring-red-300 dark:focus:ring-red-800 font-medium rounded-lg text-sm inline-flex items-center px-5 py-2.5 text-center mr-2" wire:click="restore(' . $row->id . ')">
                                                ' . __('messages.Yes, I am sure') . '
                                            </button>
                                            <button data-modal-toggle="restore-modal" type="button" class="text-gray-500 bg-white hover:bg-gray-100 focus:ring-4 focus:outline-none focus:ring-gray-200 rounded-lg border border-gray-200 text-sm font-medium px-5 py-2.5 hover:text-gray-900 focus:z-10 dark:bg-gray-700 dark:text-gray-300 dark:border-gray-500 dark:hover:text-white dark:hover:bg-gray-600 dark:focus:ring-gray-600">
                                            ' . __('messages.No, cancel') . '
                                            </button>
                                        </div>
                                    </div>
                                </div>
                            </div>';
                        return $restore . $permanently_delete;
                    }
                )->html(),
        ];


        $result = array_merge($colssums, $final_colums);
        return $result;
    }
}
